import aiohttp


class Dispatcher:
    def __init__(self, event: dict, token: str, text: str, user_id: int, peer_id: int = None, post_id: int = None, owner_id: int = None):
        self.token = token
        self.user_id = user_id
        self.peer_id = peer_id
        self.post_id = post_id
        self.owner_id = owner_id
        self.event = event
        self.text = text

    async def answer(self, text: str, attachment: str = None, keyboard: str = None) -> None:
        params = f"user_id={self.user_id}&message={text}&random_id=0"
        if attachment is not None:
            params += f"&attachment={attachment}"
        if keyboard is not None:
            params += f"keyboard={keyboard}"
        params += f"&access_token={self.token}&v=5.126"

        async with aiohttp.ClientSession() as session:
            await session.post("https://api.vk.com/method/messages.send", params=params)

    async def send_message(self, text: str, attachment: str = None, keyboard: str = None) -> None:
        params = f"peer_id={self.peer_id}&message={text}&random_id=0"
        if attachment is not None:
            params += f"&attachment={attachment}"
        if keyboard is not None:
            params += f"&keyboard={keyboard}"
        params += f"&access_token={self.token}&v=5.126"

        async with aiohttp.ClientSession() as session:
            await session.post("https://api.vk.com/method/messages.send", params=params)

    async def send_comment(self, text: str, attachment: str = None) -> None:
        params = f"owner_id={self.owner_id}&post_id={self.post_id}&message={text}"
        if attachment is not None:
            params += f"&attachment={attachment}"
        params += f"&access_token={self.token}&v=5.126"

        async with aiohttp.ClientSession() as session:
            await session.post("https://api.vk.com/method/wall.createComment", params=params)

    async def mark_as_read(self) -> None:
        params = f"peer_id={self.peer_id}&access_token={self.token}&v=5.126"

        async with aiohttp.ClientSession() as session:
            await session.post("https://api.vk.com/method/messages.markAsRead", params=params)

    async def set_typing_status(self, typing_status: str = "typing") -> None:
        params = f"peer_id={self.peer_id}&type={typing_status}&access_token={self.token}&v=5.126"

        async with aiohttp.ClientSession() as session:
            await session.post("https://api.vk.com/method/messages.setActivity", params=params)



import json


def get_button(label, color, payload=""):
    return {
        "action": {
            "type": "text",
            "payload": json.dumps(payload),
            "label": label
        },
        "color": color
    }


def get_keyboard(buttons, inline=False, one_time=True):
    keyboard = {
        "buttons": [
            [
                get_button(label, color) for label, color in button
            ] for button in buttons
        ]
    }

    if inline:
        keyboard["inline"] = inline
    else:
        keyboard["one_time"] = one_time

    keyboard = json.dumps(keyboard, ensure_ascii=False).encode("UTF-8")
    keyboard = str(keyboard.decode("UTF-8"))

    return keyboard



from typing import Union


class Condition:
    def __init__(self, command: str = None, contains_command: str = None,
                 user_id: int = None, peer_id: int = None, post_id: int = None, owner_id: int = None):
        self.command = command
        self.contains_command = contains_command
        self.user_id = user_id
        self.peer_id = peer_id
        self.post_id = post_id
        self.owner_id = owner_id

    def new_event(self, text: str = None, user_id: int = None, peer_id: int = None, post_id: int = None, owner_id: int = None) -> bool:
        if (self.contains_command is not None and self.contains_command in text) or \
                (self.command is not None and self.command == text) or \
                (self.user_id is not None and self.user_id == user_id) or \
                (self.peer_id is not None and self.peer_id == peer_id) or \
                (self.post_id is not None and self.post_id == post_id) or \
                (self.owner_id is not None and self.owner_id == owner_id):
            return True

        return False


class And:
    def __init__(self, *conditions: Union[Condition, "And", "Or"]):
        self.conditions = conditions

    def new_event(self, text: str = None, user_id: int = None, peer_id: int = None, post_id: int = None, owner_id: int = None) -> bool:
        if all(list(map(lambda condition: condition.new_event(text, user_id, peer_id), self.conditions))):
            return True

        return False


class Or:
    def __init__(self, *conditions: Union[Condition, And, "Or"]):
        self.conditions = conditions

    def new_event(self, text: str = None, user_id: int = None, peer_id: int = None, post_id: int = None, owner_id: int = None) -> bool:
        if any(list(map(lambda condition: condition.new_event(text, user_id, peer_id), self.conditions))):
            return True

        return False



from typing import Union

import asyncio
import aiohttp

from .dispatcher import Dispatcher
from .condition import Condition, And, Or


class Handler:
    def __init__(self, condition: Union[Condition, And, Or] = None, is_lower: bool = False, func=None):
        self.condition = condition
        self.is_lower = is_lower
        self.func = func
        self.is_ready = True

    def __call__(self, func):
        async def wrapper(event: dict, token: str, text: str = None, user_id: int = None, peer_id: int = None, owner_id: int = None, post_id: int = None):
            await func(Dispatcher(event, token, text, user_id, peer_id=peer_id, post_id=post_id, owner_id=owner_id))

        return type(self)(self.condition, self.is_lower, wrapper)


class CustomHandler(Handler):
    def __call__(self, func):
        async def wrapper(event: dict, token: str, text: str = None, user_id: int = None, peer_id: int = None, owner_id: int = None, post_id: int = None):
            await func(Dispatcher(event, token, text, user_id, peer_id=peer_id, post_id=post_id, owner_id=owner_id))

        handler = CustomHandler(self.condition, self.is_lower, wrapper)
        handler.event_type = self.event_type
        return handler

    async def new_event(self, token: str, event: dict) -> None:
        if "message" in event["object"]:
            text = event["object"]["message"]["text"]
            user_id = event["object"]["message"]["from_id"]
            peer_id = event["object"]["message"]["peer_id"]
            post_id = None
            owner_id = None
        else:
            text = event["object"]["text"]
            user_id = event["object"]["from_id"]
            peer_id = event["object"].get("peer_id")
            post_id = event["object"].get("post_id")
            owner_id = event["object"].get("owner_id")

        while True:
            if self.is_ready:
                self.is_ready = False
                await self.func(event, token, text, user_id, peer_id=peer_id, owner_id=owner_id, post_id=post_id)
                self.is_ready = True
                break
            else:
                await asyncio.sleep(0.5)


class MessageNewHandler(Handler):
    event_type = "message_new"

    async def new_event(self, token: str, event: dict) -> None:
        try:
            text = event["object"]["message"]["text"] if not self.is_lower else event["object"]["message"]["text"].lower()
            peer_id = event["object"]["message"]["peer_id"]
            user_id = event["object"]["message"]["from_id"]

            if self.condition.new_event(text=text, user_id=user_id, peer_id=peer_id):
                while True:
                    if self.is_ready:
                        self.is_ready = False
                        await self.func(event, token, text, user_id, peer_id=peer_id)
                        self.is_ready = True
                        break
                    else:
                        await asyncio.sleep(0.5)

        except (IndexError, KeyError):
            pass


class MessageEditHandler(Handler):
    event_type = "message_edit"

    async def new_event(self, token: str, event: dict) -> None:
        try:
            text = event["object"]["text"] if not self.is_lower else event["object"]["text"].lower()
            peer_id = event["object"]["peer_id"]
            user_id = event["object"]["from_id"]

            if self.condition.new_event(text=text, user_id=user_id, peer_id=peer_id):
                while True:
                    if self.is_ready:
                        self.is_ready = False
                        await self.func(event, token, text, user_id, peer_id=peer_id)
                        self.is_ready = True
                        break
                    else:
                        await asyncio.sleep(0.5)

        except (IndexError, KeyError):
            pass


class WallReplyNewHandler(Handler):
    event_type = "wall_reply_new"

    async def new_event(self, token: str, event: dict) -> None:
        try:
            text = event["object"]["text"] if not self.is_lower else event["object"]["text"].lower()
            owner_id = event["object"]["owner_id"]
            user_id = event["object"]["from_id"]
            post_id = event["object"]["post_id"]

            if self.condition.new_event(text=text, user_id=user_id, post_id=post_id, owner_id=owner_id):
                while True:
                    if self.is_ready:
                        self.is_ready = False
                        await self.func(event, token, text, user_id, owner_id=owner_id, post_id=post_id)
                        self.is_ready = True
                        break
                    else:
                        await asyncio.sleep(0.5)

        except (IndexError, KeyError):
            pass


class WallReplyEditHandler(WallReplyNewHandler):
    event_type = "wall_reply_edit"


class WallPostNewHandler(Handler):
    event_type = "wall_post_new"

    async def new_event(self, token: str, event: dict) -> None:
        try:
            text = event["object"]["text"] if not self.is_lower else event["object"]["text"].lower()
            owner_id = event["object"]["owner_id"]
            user_id = event["object"]["from_id"]
            post_id = event["object"]["id"]

            if self.condition.new_event(text=text, user_id=user_id, post_id=post_id, owner_id=owner_id):
                while True:
                    if self.is_ready:
                        self.is_ready = False
                        await self.func(event, token, text, user_id, owner_id=owner_id, post_id=post_id)
                        self.is_ready = True
                        break
                    else:
                        await asyncio.sleep(0.5)

        except (IndexError, KeyError):
            pass


class BoardPostNewHandler(Handler):
    event_type = "board_post_new"

    async def new_event(self, token: str, event: dict) -> None:
        try:
            text = event["object"]["text"] if not self.is_lower else event["object"]["text"].lower()
            owner_id = event["group_id"]
            user_id = event["object"]["from_id"]
            post_id = event["object"]["topic_id"]

            if self.condition.new_event(text=text, user_id=user_id, post_id=post_id, owner_id=owner_id):
                while True:
                    if self.is_ready:
                        self.is_ready = False
                        await self.func(event, token, text, user_id, owner_id=owner_id, post_id=post_id)
                        self.is_ready = True
                        break
                    else:
                        await asyncio.sleep(0.5)

        except (IndexError, KeyError):
            pass


class BoardPostEditHandler(BoardPostNewHandler):
    event_type = "board_post_edit"


class Handlers:
    message_new = MessageNewHandler
    message_edit = MessageEditHandler
    wall_reply_new = WallReplyNewHandler
    wall_reply_edit = WallReplyEditHandler
    wall_post_new = WallPostNewHandler
    board_post_new = BoardPostNewHandler
    board_post_edit = BoardPostEditHandler

    def __new__(cls, event_type):
        handler = CustomHandler()
        handler.event_type = event_type
        return handler



import asyncio
import aiohttp

from .handlers import Handlers


class Handler:
    on = Handlers


class Bot:
    def __init__(self, token: str, group_id: int):
        self.token = token
        self.group_id = group_id
        self.config = {}
        self.handlers = []

    def handle(self, handler: Handler) -> None:
        self.handlers.append(handler)

    async def run_polling(self) -> None:
        async with aiohttp.ClientSession() as session:
            async with session.get("https://api.vk.com/method/groups.getLongPollServer",
                                   params=f"group_id={self.group_id}&access_token={self.token}&v=5.126") as response:
                config = await response.json()
                print(config)
                self.config = config["response"]
                self.config["server"] = self.config["server"].replace("https://", "")

                async with session.get("https://api.vk.com/method/groups.getLongPollSettings",
                                       params=f"group_id={self.group_id}&access_token={self.token}&v=5.126") as response:
                    print(await response.json())

                while True:
                    try:
                        async with session.get("https://%s?act=a_check&key=%s&ts=%s&wait=60&mode=2&version=5.126" % (self.config["server"], self.config["key"], self.config["ts"])) as response:
                            event = await response.json()
                            updates = event["updates"]
                            print(event)
                            self.config["ts"] = event["ts"]

                            tasks = [asyncio.create_task(handler.new_event(self.token, updates[0]))
                                     for handler in self.handlers if handler.event_type == updates[0]["type"]]

                    except:
                        pass

    async def execute(self, method: str, **params) -> dict:
        query = f"https://api.vk.com/method/{method}?"
        for key, value in params.items():
            query += f"{key}={value}&"
        query += f"access_token={self.token}&v=5.126"

        async with aiohttp.ClientSession() as session:
            async with session.get(query) as response:
                return await response.json()


def run_polling(bot: Bot) -> None:
    loop = asyncio.get_event_loop()
    loop.run_until_complete(bot.run_polling())



from setuptools import setup, find_packages


requirements = ["asyncio>=3", "aiohttp>=3"]


setup(
    name="async_VKsher",
    version="1.0.0",
    author="Kulenov Islam",
    author_email="kit.werr34@gmail.com",
    description="asyncVK is asynchronous library for creating a bot in VK",
    url="https://github.com/Ekventor/asyncVK",
    packages=find_packages(),
    install_requirement=requirements,
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ]
)
